<?php
/*---------------------------------------------------------------------------------------------
Plugin Name: GetWeather
Plugin URI: http://dev.wp-plugins.org/wiki/GetWeather
Author: Jeff Minard
Author URI: http://thecodepro.com/
Version: 1.2.1
Description: Shows current weather conditions by querying the weather.com website.
Date: 22 January 2005
Extra Info: Icon set "Crystal" care of Daniel Brauer's (DockZone)
            Extra icons can be downloaded from 
            http://www.samurize.com/modules/ipboard/index.php?s=f171e96010a2120b73cd108b18c02203&showtopic=3857

ChangeLog:
	1.0.0 - Original
	1.2.0 - Added support for simple-cache ( cache_store() and cache_recall() ) 
			to speed things up a bit.
	      - Changed the hack for IE. Caching would detect one browser, and 
	      	cache the wrong version for everyone else. Now instead of an img tag, 
	      	the "img" is placed via background styles in a DIV that will work in 
	      	compliant browsers, and will also work with IE.
	1.2.1 - A few simple changes
	      - Added simple-cache.php to the package.
	
Instructions:

1. Change the config variables below in the "EDIT THIS" section
2. Upload this plugin to your wp-plugins/ directory
3. Active the GetWeather Plugin via the admin menu
4. Place this code in your template:

    <ul><?php get_weather('94115') ?></ul>

    Or get much more fancy and include:

    <ul><?php get_weather('94115', 'SF, CA', 'icon,temp,high,low') ?></ul>

You need only change the "citycode" (the first variable in the 
function) to your own city. You may use a US zip code in place 
of a standard city code. International users will have to use 
a standard city code.


Extra Functionality:

	<?php get_weather($citycode, $title, $options) ?>

$citycode 
	A required attribute to define what city you want weather from
$title
	If using the attribute "desc", this string will replace the
	default city name.
$options
	A list of what attributes you want the plugin to return, and 
	in what order to return them in.


Currently, you can include these things in your list of options (in any order, too):

	desc - descriptiong - either the default, or one you define 
	icon - display the weather icon
	temp - temperature
	forecast - text description of current conditions
	curtime - current time (or last weather update time)
	sunrise - sunrise for today
	sunset - sunset for today
	vis - visible distance
	wind - wind speed
	hum - humidity
	dew - dew point
	high - high temperature
	low - low temperature
	sunset-sunrise - sunset/rise as one item
	high-low - High and low as one item

---------------------------------------------------------------------------------------------*/

//
// EDIT THESE:
//

$measurement_system = 's'; // 's' or 'm' (US Standard or Metric)
$iconDirURL = "http://YOURSERVER.com/iconset"; // no trailing slash, please
$use_cache = true; /* 
					if you have simple-caches.php installed, setting this to
					'true' will attempt to use the caching mechanisms.
					*/

// 
// No need to edit further!
//

function get_weather($citycode='USCA0987', $desc='', $order="desc,icon,temp,forecast,curtime,sunrise,sunset,vis,wind,hum,dew,high,low") {
	// use this function just to echo out the LI's
	echo( get_weather_raw($citycode, $desc, $order) );
}

function get_weather_raw($citycode='USCA0987', $desc='', $order="desc,icon,temp,forecast,curtime,sunrise,sunset,vis,wind,hum,dew,high,low") {
	// this function does the heavy lifting and simple "returns" the li string.
	// use this if you want to do something to the string before echoing.
	// Get the weather from xoap weather.com and parse it to an array

	global $iconDirURL, $measurement_system, $transparent_hack, $use_cache;

	if( !function_exists(cache_recall) || !function_exists(cache_store) ) {
		// caching function not available
		$use_cache = false;
	}

	// check the cache 1.2
	if($use_cache) {
		$function_string = "get_weather_raw($citycode,$desc,$order)";
		if($returns = cache_recall($function_string)) {
			return $returns;
		}
	}


	$file = "http://xoap.weather.com/weather/local/$citycode?cc=*&dayf=1&unit=$measurement_system";
	$xml_parser = xml_parser_create();

	if (!($fp = fopen($file, "r"))) {
	   return("<li>Weather Not Available (Read Error)</li>");
	}
	
	$data = '';
	while (!feof($fp)) {
	  $data .= fread($fp, 8192);
	}

	fclose($fp);
	xml_parse_into_struct($xml_parser, $data, $vals, $index);
	xml_parser_free($xml_parser);

	$params = array();
	$level = array();
	foreach ($vals as $xml_elem) {
	  if ($xml_elem['type'] == 'open') {
	   if (array_key_exists('attributes',$xml_elem)) {
		 list($level[$xml_elem['level']],$extra) = array_values($xml_elem['attributes']);
	   } else {
		 $level[$xml_elem['level']] = $xml_elem['tag'] ;
	   }
	  }
	  if ($xml_elem['type'] == 'complete') {
	   $start_level = 1;
	   $php_stmt = '$params';
	   while($start_level < $xml_elem['level']) {
		 $php_stmt .= '[$level['.$start_level.']]';
		 $start_level++;
	   }
	   $php_stmt .= '[$xml_elem[\'tag\']] = $xml_elem[\'value\'];';
	   eval($php_stmt);
	  }
	}

	// mmm, parsing goodness
	//
	// Now, return it in the order requested

	$order_array = explode(',', $order);

	foreach($order_array as $val) {

		switch($val) {
			case 'desc':
				if($desc != '') { $d = $desc; } else { $d = $params["2.0"][CC][OBST]; }
				$output .= lir("Weather for $d", $val);
				break;

			case 'icon':
				$icon = $iconDirURL .'/' . $params["2.0"][CC][ICON] . '.png';

				$s = "<div id=\"weather-image\" style=\"background: url('$icon') !important; background: transparent;	filter: progid:DXImageTransform.Microsoft.AlphaImageLoader(src='$icon', sizingMethod='scale'); height: 128px; width: 128px;\"> </div>";
				
				$output .= lir($s, $val);
				break;

			case 'temp':
				$output .= lir("Temperature: " . $params["2.0"][CC][TMP] . $params["2.0"][HEAD][UT], $val);
				break;
			
			case 'forecast':
				$output .= lir("Forecast: " . $params["2.0"][CC][T], $val);
				break;

			case 'curtime':
				$output .= lir("Current Time: " . $params["2.0"][$citycode]["TM"], $val);
				break;
			
			case 'sunrise':
				$output .= lir("Sunrise: " . $params["2.0"][$citycode][SUNR], $val);
				break;
			
			case 'sunset':
				$output .= lir("Sunset: " . $params["2.0"][$citycode][SUNS], $val);
				break;
			
			case 'sunrise-sunset':
				$output .= lir("Sunrise/Sunset: " . $params["2.0"][$citycode][SUNR].'/'.$params["2.0"][$citycode][SUNS], $val);
				break;

			case 'vis':
				$output .= lir("Visibility: " . $params["2.0"][CC][VIS] . $params["2.0"][HEAD][UD], $val);
				break;
			
			case 'wind':
				$output .= lir("Wind: " . $params["2.0"][CC][WIND][S] . $params["2.0"][HEAD][US], $val);
				break;
			
			case 'hum':
				$output .= lir("Humidity: " . $params["2.0"][CC][HMID], $val);
				break;
			
			case 'dew':
				$output .= lir("Dewpoint: " . $params["2.0"][CC][DEWP], $val);
				break;

			case 'high':
				$output .= lir("High: " . $params["2.0"][DAYF][0][HI], $val);
				break;
			
			case 'low':
				$output .= lir("Low: " . $params["2.0"][DAYF][0][LOW], $val);
				break;

			case 'high-low':
				$output .= lir("High/Low: " . $params["2.0"][DAYF][0][HI].'/'.$params["2.0"][DAYF][0][LOW], $val);
				break;
		}
	}


	// store the results so that they can be recalled via cache 1.2
	if($use_cache) {
		cache_store($function_string, $output);
	}


	return $output;

}

//general function for LI'ing things.
function lir($s, $c=''){ 
	if($c!='') { $c=" class=\"$c\""; } 
	return "<li$c>$s</li>\n"; 
}

?>